@@ -1,3 +1,4 @@
+
 ###################################################################
 #  Numexpr - Fast numerical array expression evaluator for NumPy.
 #
@@ -14,6 +15,7 @@ import sys
 import platform
 import warnings
 from contextlib import contextmanager
+import subprocess
 
 import numpy as np
 from numpy import (
@@ -41,6 +43,7 @@ if sys.version_info[0] >= 3:
 # Recommended minimum versions
 from distutils.version import LooseVersion
 minimum_numpy_version = LooseVersion('1.7.0')
+present_numpy_version = LooseVersion(np.__version__)
 
 class test_numexpr(TestCase):
     """Testing with 1 thread"""
@@ -319,7 +322,6 @@ class test_numexpr(TestCase):
         # Check for issue #313, whereby clearing f_locals also clear f_globals
         # if in the top-frame. This cannot be done inside `unittest` as it is always 
         # executing code in a child frame.
-        import subprocess
         script = r';'.join([
                 r"import numexpr as ne",
                 r"a=10",
@@ -333,7 +335,7 @@ class test_numexpr(TestCase):
                 r"a += 1",
             ])
         # Raises CalledProcessError on a non-normal exit
-        check = subprocess.check_call('{0} -c "{1}"'.format(sys.executable, script), shell=True)
+        check = subprocess.check_call([sys.executable, '-c', script])
         # Ideally this test should also be done against ipython but it's not 
         # a requirement.
 
@@ -505,7 +507,7 @@ class test_evaluate(TestCase):
         # Execution order set here so as to not use too many threads
         # during the rest of the execution.  See #33 for details.
         def test_changing_nthreads_00_inc(self):
-            a = linspace(-1, 1, 1e6)
+            a = linspace(-1, 1, 1000000)
             b = ((.25 * a + .75) * a - 1.5) * a - 2
             for nthreads in range(1, 7):
                 numexpr.set_num_threads(nthreads)
@@ -513,7 +515,7 @@ class test_evaluate(TestCase):
                 assert_array_almost_equal(b, c)
 
         def test_changing_nthreads_01_dec(self):
-            a = linspace(-1, 1, 1e6)
+            a = linspace(-1, 1, 1000000)
             b = ((.25 * a + .75) * a - 1.5) * a - 2
             for nthreads in range(6, 1, -1):
                 numexpr.set_num_threads(nthreads)
@@ -810,9 +812,14 @@ class test_strings(TestCase):
         str_list = [
             b'\0\0\0', b'\0\0foo\0', b'\0\0foo\0b', b'\0\0foo\0b\0',
             b'foo\0', b'foo\0b', b'foo\0b\0', b'foo\0bar\0baz\0\0']
+        min_tobytes_version = LooseVersion('1.9.0')
         for s in str_list:
             r = evaluate('s')
-            self.assertEqual(s, r.tostring())  # check *all* stored data
+            if present_numpy_version >= min_tobytes_version:
+                self.assertEqual(s, r.tobytes())  # check *all* stored data
+            else:
+                # ndarray.tostring() is deprecated as of NumPy 1.19
+                self.assertEqual(s, r.tostring())  # check *all* stored data
 
     def test_compare_copy(self):
         sarr = self.str_array1
@@ -956,17 +963,56 @@ def _environment(key, value):
         else:
             del os.environ[key]
 
-
 # Test cases for the threading configuration
 class test_threading_config(TestCase):
+    def test_max_threads_unset(self):
+        # Has to be done in a subprocess as `importlib.reload` doesn't let us 
+        # re-initialize the threadpool
+        script = '\n'.join([
+                "import os",
+                "if 'NUMEXPR_MAX_THREADS' in os.environ: os.environ.pop('NUMEXPR_MAX_THREADS')",
+                "if 'OMP_NUM_THREADS' in os.environ: os.environ.pop('OMP_NUM_THREADS')",
+                "import numexpr",
+                "assert(numexpr.nthreads <= 8)",
+                "exit(0)"])
+        subprocess.check_call([sys.executable, '-c', script])
+
+    def test_max_threads_set(self):
+        # Has to be done in a subprocess as `importlib.reload` doesn't let us 
+        # re-initialize the threadpool
+        script = '\n'.join([
+                "import os",
+                "os.environ['NUMEXPR_MAX_THREADS'] = '4'",
+                "import numexpr",
+                "assert(numexpr.MAX_THREADS == 4)",
+                "exit(0)"])
+        subprocess.check_call([sys.executable, '-c', script])
+
     def test_numexpr_num_threads(self):
         with _environment('OMP_NUM_THREADS', '5'):
+            # NUMEXPR_NUM_THREADS has priority
             with _environment('NUMEXPR_NUM_THREADS', '3'):
-                self.assertEqual(3, numexpr.detect_number_of_threads())
+                 if 'sparc' in platform.machine():
+                     self.assertEqual(1, numexpr._init_num_threads())
+                 else:
+                     self.assertEqual(3, numexpr._init_num_threads())
 
     def test_omp_num_threads(self):
         with _environment('OMP_NUM_THREADS', '5'):
-            self.assertEqual(5, numexpr.detect_number_of_threads())
+            if 'sparc' in platform.machine():
+                self.assertEqual(1, numexpr._init_num_threads())
+            else:
+                self.assertEqual(5, numexpr._init_num_threads())
+
+    def test_vml_threads_round_trip(self):
+        n_threads = 3
+        if use_vml:
+            numexpr.utils.set_vml_num_threads(n_threads)
+            set_threads = numexpr.utils.get_vml_num_threads()
+            self.assertEqual(n_threads, set_threads)
+        else:
+            self.assertIsNone(numexpr.utils.set_vml_num_threads(n_threads))
+            self.assertIsNone(numexpr.utils.get_vml_num_threads())
 
 
 # Case test for threads
@@ -1042,7 +1088,7 @@ def print_versions():
 
     np_version = LooseVersion(np.__version__)
 
-    if minimum_numpy_version < np_version:
+    if np_version < minimum_numpy_version:
         print('*Warning*: NumPy version is lower than recommended: %s < %s' % (np_version, minimum_numpy_version))
     print('-=' * 38)
     print('Numexpr version:   %s' % numexpr.__version__)
